/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include <ctime>
#include <algorithm>
#include "funticket.h"
#include "core/misc.h"



namespace Ticket
{



FunTicket::FunTicket()
{
	fun.url = L"ticket";
}


void FunTicket::SetTDb(TDb * ptdb)
{
	tdb = ptdb;
}


void FunTicket::SetTicketInfo(TicketInfo * pinfo)
{
	ticket_info = pinfo;
}


void FunTicket::Clear()
{
	ticket_info->Clear();
}


bool FunTicket::HasAccess()
{
	return !request->is_item;
}





void FunTicket::MakeGet()
{
	ticket_info->FindCurrentConf();
	ticket_info->ReadTicket(request->dir_tab.back()->id);

	DbItemQuery iq;
	iq.WhereParentId(request->dir_tab.back()->id);
	iq.WhereType(Item::file);
	iq.WhereAuth(Item::auth_none);
	db->GetItems(request->item_tab, iq);

	if( !request->item_tab.empty() )
	{
		if( system->HasReadAccess(request->item_tab[0]) )
		{
			ticket_info->item = request->item_tab[0];
			request->item_tab.erase(request->item_tab.begin());
		}
		else
		{
			request->status = WINIX_ERR_PERMISSION_DENIED;
			return;
		}
	}

	system->CheckAccessToItems(request->item_tab);
	SelectTickets();
}



void FunTicket::SelectTickets()
{
	child_tab.clear();
	system->dirs.GetDirChilds(request->dir_tab.back()->id, child_tab);

	SortDirs();
	PageDirs();
	CreateIdTab();
	tdb->GetTickets(id_tab, ticket_info->ticket_tab);
	ticket_info->SortTickets();
}



bool FunTicket::SortByDate::operator()(const Item * item1, const Item * item2)
{
	tm tm1 = item1->date_creation;
	tm tm2 = item2->date_creation;

	time_t time1 = Time(tm1);
	time_t time2 = Time(tm2);

return time1 < time2;
}



void FunTicket::SortDirs()
{
	// temporarily sorting by date
	SortByDate sort(this);
	std::sort(child_tab.begin(), child_tab.end(), sort);
}



void FunTicket::PageDirs()
{
	// temporarily show every tickets
}



void FunTicket::CreateIdTab()
{
	id_tab.clear();

	for(size_t i=0 ; i<child_tab.size() ; ++i)
		id_tab.push_back(child_tab[i]->id);
}


} // namespace
