/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include "tdb.h"
#include "funticket.h"
#include "editticket.h"
#include "createticket.h"
#include "core/log.h"
#include "core/plugin.h"


extern "C" void Init(PluginInfo &);


namespace Ticket
{
const wchar_t plugin_name[] = L"ticket";

TDb tdb;
TicketInfo ticket_info;

FunTicket    fun_ticket;
CreateTicket fun_create_ticket;
EditTicket   fun_edit_ticket;



void AddFunctions(PluginInfo & info)
{
	info.functions->Add(fun_ticket);
	info.functions->Add(fun_create_ticket);
	info.functions->Add(fun_edit_ticket);
}


void SelectDefaultFunction(PluginInfo & info)
{
	if( info.request->is_item )
		return;

	if( info.system->mounts.pmount->type == ticket_info.mount_type_ticket )
		info.request->function = &fun_ticket;
}


void AddMounts(PluginInfo & info)
{
	Mounts & m = info.system->mounts;
	
	ticket_info.mount_type_ticket					= m.AddMountType(L"ticket");
	ticket_info.mount_par_ticket					= m.AddMountPar(L"ticket");
	ticket_info.mount_par_ticket_conf				= m.AddMountPar(L"ticket_conf");

	// !! bedzie do usuniecia
	ticket_info.mount_par_ticket_type				= m.AddMountPar(L"ticket_type");
	ticket_info.mount_par_ticket_type_default		= m.AddMountPar(L"ticket_type_default");
	ticket_info.mount_par_ticket_status				= m.AddMountPar(L"ticket_status");
	ticket_info.mount_par_ticket_status_default		= m.AddMountPar(L"ticket_status_default");
	ticket_info.mount_par_ticket_priority			= m.AddMountPar(L"ticket_priority");
	ticket_info.mount_par_ticket_priority_default	= m.AddMountPar(L"ticket_priority_default");
	ticket_info.mount_par_ticket_category			= m.AddMountPar(L"ticket_category");
	ticket_info.mount_par_ticket_category_default	= m.AddMountPar(L"ticket_category_default");
	ticket_info.mount_par_ticket_expected			= m.AddMountPar(L"ticket_expected");
	ticket_info.mount_par_ticket_expected_default	= m.AddMountPar(L"ticket_expected_default");
	ticket_info.mount_par_createticket_on			= m.AddMountPar(L"createticket_on");
}




void FstabChanged(PluginInfo & info)
{
	ticket_info.ReadTicketConf(true);
}


void ProcessRequest(PluginInfo & info)
{
	if( info.request->function == &info.functions->fun_reload )
	{
		if( info.request->IsParam(L"tickets") )
			ticket_info.ReadTicketConf();
	}
}


void RemoveTicket(PluginInfo & i)
{
	tdb.RemoveTicket(i.l1);
}


void InitTicket(PluginInfo & info)
{
	ticket_info.progress_prefix = info.config->Text(L"ticket_form_progress_prefix", L"progress");
}

void AddEzcFunctions(PluginInfo & info);

} // namespace Ticket



void Init(PluginInfo & info)
{
using namespace Ticket;

	plugin.Assign(WINIX_PLUGIN_INIT,                InitTicket);
	plugin.Assign(WINIX_TEMPLATES_CREATEFUNCTIONS,	AddEzcFunctions);
	plugin.Assign(WINIX_CREATE_FUNCTIONS,			AddFunctions);
	plugin.Assign(WINIX_SELECT_DEFAULT_FUNCTION,	SelectDefaultFunction);
	plugin.Assign(WINIX_ADD_MOUNTS,					AddMounts);
	plugin.Assign(WINIX_FSTAB_CHANGED,				FstabChanged);
	plugin.Assign(WINIX_PROCESS_REQUEST,			ProcessRequest);
	plugin.Assign(WINIX_DIR_REMOVED,				RemoveTicket);

	tdb.SetConn(info.db->GetConn());
	tdb.LogQueries(info.config->log_db_query);

	ticket_info.SetDb(info.db);
	ticket_info.SetTDb(&tdb);
	ticket_info.SetSystem(info.system);
	ticket_info.SetRequest(info.request);

	fun_ticket.SetTDb(&tdb);
	fun_ticket.SetTicketInfo(&ticket_info);

	fun_edit_ticket.SetTDb(&tdb);
	fun_edit_ticket.SetTicketInfo(&ticket_info);

	fun_create_ticket.SetTDb(&tdb);
	fun_create_ticket.SetTicketInfo(&ticket_info);

	info.p1 = (void*)plugin_name;
}


